pub enum DoDefaultOutput {
DoDefault,
Override,
}Expand description
The return value of argument callbacks.
Decide if the argument callback should emit the default output or if it should emit no output at all.
Variants
DoDefault
Emit the default output.
The user may still emit output before the default behavior.
Override
Do not emit the default output for this argument.