pub enum LookatCount {
N1 = 1,
N2 = 2,
}Expand description
How many macro packets the current macro actually refers to.
Variants
N1 = 1
This macro does not correspond to a complete gsSPLookAt, but instead
is a singular use of either gsSPLookAtX or gsSPLookAtY.
N2 = 2
This macro is a complete gsSPLookAt macro.